#include "clientnetworking.h"

ClientNetworking::ClientNetworking(QObject *parent, QString randomID, QString name, char* IP, int PORT) : QThread(parent)
{
    this->randomID = randomID;
    this->name = name;
    this->IP = IP;
    this->PORT=PORT;
}

QString ClientNetworking::getMetrics(QString randomID, QString name){
    QString result;

    // add the generated ID to the result
    result.append(randomID + ";");
    // add the name
    result.append(name + ";");
    // add the window title
    WCHAR windowTitle[128];
    GetWindowTextW(GetForegroundWindow(), windowTitle, 128);
    result.append(QString::fromWCharArray(windowTitle) + ";");
    // add the window ID number
    DWORD WindowID;
    GetWindowThreadProcessId(GetForegroundWindow(), &WindowID);
    result.append(QString::number(WindowID) + ";");
    // add the information about the number of monitors
    result.append(QString::number(GetSystemMetrics(SM_CMONITORS)) + ";");
    // add checking if the application is running in a remote session
    result.append(QString::number(GetSystemMetrics(SM_REMOTESESSION)) + ";");
    // add the computer's manufacturer and model to the result
    wchar_t *manufacturer = new wchar_t[255];
    wchar_t *model = new wchar_t[255];
    DWORD length = 1024;
    RegGetValue(HKEY_LOCAL_MACHINE, L"SYSTEM\\CurrentControlSet\\Control\\SystemInformation", L"SystemManufacturer", RRF_RT_ANY, NULL, manufacturer, &length);
    result.append(QString::fromWCharArray(manufacturer)+", ");
    RegGetValue(HKEY_LOCAL_MACHINE, L"SYSTEM\\CurrentControlSet\\Control\\SystemInformation", L"SystemProductName", RRF_RT_ANY, NULL, model, &length);
    result.append(QString::fromWCharArray(model)+";");
    // add the timestamp in the form of seconds since Epoch
    result.append(QString::number(QDateTime::currentSecsSinceEpoch())+";");
    // add the list of running processes
    PROCESSENTRY32 entry;
    entry.dwSize = sizeof(PROCESSENTRY32);
    HANDLE snapshot = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
    QString checkRepeat;
    while(Process32Next(snapshot, &entry)){
        if(checkRepeat != QString::fromWCharArray(entry.szExeFile)){
        result.append(entry.szExeFile);
        result.append(",");
        checkRepeat = QString::fromWCharArray(entry.szExeFile);
        }
    }
    CloseHandle(snapshot);
    result.chop(1);

    return result;
}

void ClientNetworking::run(){
    running=true;
    // initialize Windows Sockets 2
    emit updateStatus("<p style=\"background-color:Red;\">&nbsp; Initializing &nbsp;</p>");
    WSADATA winsock;
    if (WSAStartup(514, &winsock) != 0){
        emit updateStatus("<p style=\"background-color:Red;\">&nbsp; Network Error &nbsp;</p>");
        this->terminate();
    }
    SOCKET clientSocket = socket(AF_INET, SOCK_STREAM, 0);
    if (clientSocket == INVALID_SOCKET){
        emit updateStatus("<p style=\"background-color:Red;\">&nbsp; Network Error &nbsp;</p>");
        this->terminate();
    }
    struct sockaddr_in init;
    init.sin_port = htons(PORT);
    init.sin_family = AF_INET;
    init.sin_addr.s_addr = inet_addr(IP);
    // connect to the server
    emit updateStatus("<p style=\"background-color:Yellow;\">&nbsp; Connecting &nbsp;</p>");
    int checkConnection = WSAAPI::connect(clientSocket, (sockaddr*)&init, sizeof(init));
    while (checkConnection == SOCKET_ERROR && running){
        closesocket(clientSocket);
        clientSocket = socket(AF_INET, SOCK_STREAM, 0);
        checkConnection = WSAAPI::connect(clientSocket, (sockaddr*)&init, sizeof(init));
    }
    // initialize OpenSSL with TLS 1.3
    SSL *ssl;
    SSL_library_init();
    SSLeay_add_ssl_algorithms();
    SSL_load_error_strings();
    SSL_CTX *sslctx = SSL_CTX_new(TLS_method());
    SSL_CTX_set_min_proto_version(sslctx, TLS1_3_VERSION);
    ssl = SSL_new(sslctx);
    SSL_set_fd(ssl, clientSocket);
    SSL_connect(ssl);
    emit updateStatus("<p style=\"background-color:#5BC85B;\">&nbsp; Connected &nbsp;</p>");
    status = 1;
    while(running){
        msleep(250);
        // collect the metrics and write them into result variable
        QString result = getMetrics(randomID, name);
        // make sure the message fits
        if (result.length()>8192)
            result.chop(result.length()-8192);
        char temp[8192];
        memset(temp, 0, 8191);
        memcpy(temp, result.toStdString().c_str() ,result.size());
        // send the metrics to the server over TLS
        int checkSSLwrite = SSL_write(ssl, temp, 8192);
        // if the message is not sent, indicate the loss of the connection
        if (checkSSLwrite == -1){
            emit updateStatus("<p style=\"background-color:Yellow;\">&nbsp; Connecting &nbsp;</p>");
            closesocket(clientSocket);
            WSACleanup();
            status = 0;
        }
    }
    closesocket(clientSocket);
    WSACleanup();
}
