#include "mainwindow.h"
#include "ui_mainwindow.h"

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
    , ui(new Ui::MainWindow)
{
    // make the window nonminimizeable, nonresizeable, and stays on top
    setWindowFlags(Qt::Dialog | Qt::MSWindowsFixedSizeDialogHint | Qt::WindowStaysOnTopHint);
    // generate a number between 0 and 999999
    std::mt19937 generator(std::time(nullptr));
    std::uniform_int_distribution<int> randomNumber(0,999999);
    // initialize a config file
    QSettings settings("config.ini", QSettings::IniFormat);
    // if the address was previously entered, show it in the upcoming prompt
    QString address = settings.value("IP_Address", "").toString();
    // prompt the user to enter the IP address of the server and verify the IP is valid
    bool isDigit[5];
    do{
        bool ok;
        address = QInputDialog::getText(0, "IP Address", "Enter the IP address in the form of A.B.C.D (or A.B.C.D:PORT):", QLineEdit::Normal, address, &ok);
        if (!ok)
            std::exit(EXIT_FAILURE);
        settings.setValue("IP_Address", address);
        IPandPort = address.split(':');
    } while (address.isEmpty() || IPandPort[0].split(".").length() != 4 ||
             !(IPandPort[0].split(".").at(0).toInt(&isDigit[0])>0 && IPandPort[0].split(".").at(0).toInt()<=255) || !isDigit[0] ||
             !(IPandPort[0].split(".").at(1).toInt(&isDigit[1])>=0 && IPandPort[0].split(".").at(1).toInt()<=255) || !isDigit[1] ||
             !(IPandPort[0].split(".").at(2).toInt(&isDigit[2])>=0 && IPandPort[0].split(".").at(2).toInt()<=255) || !isDigit[2] ||
             !(IPandPort[0].split(".").at(3).toInt(&isDigit[3])>=0 && IPandPort[0].split(".").at(3).toInt()<=255) || !isDigit[3]);
    // check if the port is entered
    if(IPandPort.length()>1)
        PORT = IPandPort[1].toInt();
    // if the name was previously entered, show it in the upcoming prompt
    name = settings.value("Name", "").toString();
    // prompt the user to enter the full name and verify it
    do{
        bool ok;
        name = QInputDialog::getText(0, "Your Name", "Enter your full name in the form of First Name Last Name:", QLineEdit::Normal, name, &ok);
        if (!ok)
            std::exit(EXIT_FAILURE);
        settings.setValue("Name", name);
    } while (name.isEmpty() || name.split(" ").length()<2 || name.length() > 50);
    // convert the number from 0 to 999999 into a six-digit number by adding zeroes in front
    randomID = QStringLiteral("%1").arg(randomNumber(generator), 6, 10, QLatin1Char('0'));
    // update the name and the ID in the GUI
    ui->setupUi(this);
    ui->nameLabel->setText(name);
    ui->idLabel->setText(randomID);
    // initialize a networking thread and pass it the parameters
    networkingInit = new ClientNetworking(this, randomID, name, IPandPort[0].toLocal8Bit().data(), PORT);
    // link the updateStatus signal of the clientnetworking class to the statusUpdater function of the mainwindow class
    connect(networkingInit, SIGNAL(updateStatus(QString)),this, SLOT(statusUpdater(QString)));
    networkingInit->start();
    // initialize a timer that invokes the function networkingRestart of the mainwindow class every second
    QTimer *timer = new QTimer;
    connect(timer,SIGNAL(timeout()), this, SLOT(networkingRestart()));
    timer->start(1000);
}

MainWindow::~MainWindow()
{
    networkingInit->running=false;
    Sleep(1000);
    networkingInit->terminate();
    delete ui;
}

void MainWindow::statusUpdater(QString status)
{
    // update the text of the label widget in the GUI
    ui->statusLabel->setText(status);
}

void MainWindow::networkingRestart()
{
    // restart the networking thread if it is not working or disconnected
    if (networkingInit->status==0){
        networkingInit->terminate();
        networkingInit->deleteLater();
        networkingInit = new ClientNetworking(this, randomID, name, IPandPort[0].toLocal8Bit().data(), PORT);
        connect(networkingInit, SIGNAL(updateStatus(QString)),this, SLOT(statusUpdater(QString)));
        networkingInit->start();
    }
}
