#include "networking.h"

Networking::Networking(QObject *parent, int inputPORT) : QThread(parent)
{
    PORT = inputPORT;
}

void Networking::run(){
    // initialize the OpenSSL routines
    SSL_load_error_strings();
    SSL_library_init();
    OpenSSL_add_all_algorithms();
    QVector<SSL_CTX*> SSLCTXList(10000);
    QVector<SSL*> SSLList(10000);
    // initialize the Windows Sockets 2
    WSADATA winsock;
    if (WSAStartup(MAKEWORD(2,2), &winsock) != 0){
        this->exit();
    }
    SOCKET listeningSOCKET = socket(AF_INET, SOCK_STREAM, 0);
    if (listeningSOCKET == INVALID_SOCKET){
        this->exit();
    }
    sockaddr_in init;
    init.sin_port = htons(PORT);
    init.sin_family = AF_INET;
    init.sin_addr.S_un.S_addr = INADDR_ANY;
    bind(listeningSOCKET, (sockaddr*)&init, sizeof(init));
    listen(listeningSOCKET, SOMAXCONN);
    // initialize an fd_set structure that allows simultaneous clients
    fd_set socketSet;
    FD_ZERO(&socketSet);
    FD_SET(listeningSOCKET, &socketSet);

    while (true){
        // copy an fd_set before using it
        fd_set tempSet = socketSet;
        select(0, &tempSet, NULL, NULL, NULL);
        // go through all elements of the fd_set
        for (u_int i=0; i<tempSet.fd_count; i++){
            SOCKET currentSOCKET = tempSet.fd_array[i];
            int n = currentSOCKET;
            if (currentSOCKET == listeningSOCKET){ // accepting a new client
                SOCKET newClient = accept(listeningSOCKET, NULL, NULL);
                n = newClient;
                // initialize TLS and perform a TLS handshake to establish an encrypted connection
                SSLCTXList[n] = SSL_CTX_new(TLS_method());
                SSL_CTX_set_min_proto_version(SSLCTXList[n], TLS1_3_VERSION);
                SSL_CTX_set_options(SSLCTXList[n], SSL_OP_SINGLE_DH_USE);
                // open and use a public key certificate file for TLS
                char certChar[20];
                QString cert = "cert"+QDateTime::currentDateTime().toString("yyyy")+".pem";
                memcpy(certChar, cert.toStdString().c_str() ,cert.size());
                SSL_CTX_use_certificate_file(SSLCTXList[n], certChar, SSL_FILETYPE_PEM);
                // open and use a private key certificate file for TLS
                char keyChar[20];
                QString key = "key"+QDateTime::currentDateTime().toString("yyyy")+".pem";
                memcpy(keyChar, key.toStdString().c_str() ,key.size());
                SSL_CTX_use_PrivateKey_file(SSLCTXList[n], keyChar, SSL_FILETYPE_PEM);
                // accept a TLS connection with the client
                SSLList[n] = SSL_new(SSLCTXList[n]);
                SSL_set_fd(SSLList[n], newClient);
                SSL_accept(SSLList[n]);
                FD_SET(newClient, &socketSet); // add new client to the fd_set
            } else { // receiving data from a connected client
                char temp[8192];
                memset(temp, 0, 8191);
                // read and decrypt the received message
                if (SSL_read(SSLList[n], temp, 8192)<=0){
                    // if the message is received, move on to the next client in fd_set
                    closesocket(currentSOCKET);
                    FD_CLR(currentSOCKET, &socketSet);
                }
                rxText = QString::fromLocal8Bit(temp); // save the received data in rxText variable
                if (!rxText.isEmpty()){
                    emit update(rxText); // emit an update signal if the data is correctly received
                }
            }
        }
    }
}
